@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 import os
 import time
+
 from fabric.api import local, run, cd
 from fabric import colors
 from fabric.context_managers import settings
@@ -15,13 +16,21 @@ class App(object):
     project_paths = {}
     project_package = None
     test_settings = None
+    strict = False
+    restart_command = None
+    loaddata_command = None
+    dumpdata_command = None
 
     def __init__(self, project_paths, project_package, test_settings=None,
-                 restart_command=None):
+                 strict=False, restart_command=None,
+                 loaddata_command='loaddata', dumpdata_command='dumpdata'):
         self.project_paths = project_paths
         self.project_package = project_package
         self.test_settings = test_settings
+        self.strict = strict
         self.restart_command = restart_command
+        self.loaddata_command = loaddata_command
+        self.dumpdata_command = dumpdata_command
         django.project(project_package)
 
     def run(self, command):
@@ -31,6 +40,9 @@ class App(object):
         return local("venv/bin/python manage.py %s" % command, *args, **kwargs)
 
     def run_management_command(self, instance, command):
+        if instance == 'local':
+            return self.local_management_command(command)
+
         code_dir = self.project_paths[instance]
         with cd(code_dir):
             return self.run("venv/bin/python manage.py %s" % command)
@@ -48,6 +60,8 @@ class App(object):
         if result.failed:
             print(colors.red("Tests failed"))
             if is_deploying:
+                if self.strict:
+                    abort('')
                 if not confirm('Do you really want to deploy?'):
                     abort('')
         else:
@@ -61,6 +75,14 @@ class App(object):
                 with cd(path):
                     self.run("virtualenv venv")
 
+    def syncdb(self, instance):
+        from django.conf import settings
+        if 'south' in settings.INSTALLED_APPS:
+            self.run_management_command(instance,
+                                        "syncdb --noinput --migrate")
+        else:
+            self.run_management_command(instance, "syncdb --noinput")
+
     def run_server_updates(self, instance):
         code_dir = self.project_paths[instance]
         self.check_virtualenv(code_dir)
@@ -70,13 +92,9 @@ class App(object):
 
             self.run("venv/bin/pip install -r requirements.txt")
 
-            from django.conf import settings
-            if 'south' in settings.INSTALLED_APPS:
-                self.run_management_command(instance,
-                                            "syncdb --noinput --migrate")
-            else:
-                self.run_management_command(instance, "syncdb --noinput")
+            self.syncdb(instance)
 
+            from django.conf import settings
             if 'djangobower' in settings.INSTALLED_APPS:
                 self.run_management_command(instance, "bower_install")
 
@@ -114,7 +132,7 @@ class App(object):
             self.local_management_command('compilemessages')
 
     def clone_data(self, instance):
-        dump_file = str(time.time()) + ".json"
+        dump_file = "%s.json" % str(time.time())
 
         # Ignore errors on these next steps, so that we are sure we clean up
         # no matter what
@@ -122,18 +140,20 @@ class App(object):
             # Dump the database to a file...
             self.run_management_command(
                 instance,
-                'dumpdata --all > ' + dump_file
+                '%s --all > %s' % (self.dumpdata_command, dump_file)
             )
 
             # The download that file, all uploaded files and rm the dump file
-            get(self.project_paths[instance] + dump_file, dump_file)
-            self.run('rm ' + dump_file)
+            get("%s%s" % (self.project_paths[instance], dump_file, dump_file))
+            self.run('rm %s' % dump_file)
 
-            self.local_management_command('syncdb --migrate --noinput')
-            self.local_management_command('loaddata ' + dump_file)
+            self.syncdb('local')
+
+            self.local_management_command('%s %s' % (self.loaddata_command,
+                                                     dump_file))
 
         # ... then cleanup the dump file
-        local('rm ' + dump_file)
+        local('rm %s' % dump_file)
 
     def clone_prod_data(self):
         if confirm("All local data will be replaced with prod data, OK?"):
